@@ -85,6 +85,7 @@ fn main() {
         .parse_callbacks(Box::new(bindgen::CargoCallbacks))
         .rustfmt_bindings(true)
         .no_copy("_zend_value")
+        .no_copy("_zend_string")
         .layout_tests(env::var("EXT_PHP_RS_TEST").is_ok());
 
     for binding in ALLOWED_BINDINGS.iter() {
@@ -251,6 +252,7 @@ const ALLOWED_BINDINGS: &[&str] = &[
     "IS_TYPE_REFCOUNTED",
     "IS_UNDEF",
     "IS_VOID",
+    "IS_PTR",
     "MAY_BE_ANY",
     "MAY_BE_BOOL",
     "USING_ZTS",
@@ -312,4 +314,13 @@ const ALLOWED_BINDINGS: &[&str] = &[
     "_ZEND_SEND_MODE_SHIFT",
     "_ZEND_TYPE_NULLABLE_BIT",
     "ts_rsrc_id",
+    "_ZEND_TYPE_NAME_BIT",
+    "zval_ptr_dtor",
+    "zend_refcounted_h",
+    "zend_is_true",
+    "zend_object_std_dtor",
+    "zend_std_read_property",
+    "zend_std_write_property",
+    "zend_std_get_properties",
+    "zend_std_has_property",
 ];
