@@ -38,10 +38,10 @@ impl ClassEntry {
     /// not be found or the class table has not been initialized.
     pub fn try_find(name: &str) -> Option<&'static Self> {
         ExecutorGlobals::get().class_table()?;
-        let name = ZendString::new(name, false).ok()?;
+        let mut name = ZendString::new(name, false).ok()?;
 
         unsafe {
-            crate::bindings::zend_lookup_class_ex(name.borrow_ptr(), std::ptr::null_mut(), 0)
+            crate::bindings::zend_lookup_class_ex(name.as_mut_zend_str(), std::ptr::null_mut(), 0)
                 .as_ref()
         }
     }
@@ -115,8 +115,7 @@ impl ClassEntry {
         if self.flags().contains(ClassFlags::ResolvedParent) {
             unsafe { self.__bindgen_anon_1.parent.as_ref() }
         } else {
-            let name =
-                unsafe { ZendString::from_ptr(self.__bindgen_anon_1.parent_name, false) }.ok()?;
+            let name = unsafe { self.__bindgen_anon_1.parent_name.as_ref()? };
             Self::try_find(name.as_str()?)
         }
     }
@@ -264,6 +263,11 @@ impl ClassBuilder {
     ///
     /// * `T` - The type which will override the Zend object. Must implement [`RegisteredClass`]
     /// which can be derived using the [`php_class`](crate::php_class) attribute macro.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the class name associated with `T` is not the same as the class name specified
+    /// when creating the builder.
     pub fn object_override<T: RegisteredClass>(mut self) -> Self {
         unsafe extern "C" fn create_object<T: RegisteredClass>(
             _: *mut ClassEntry,
@@ -272,6 +276,7 @@ impl ClassBuilder {
             (*ptr).get_mut_zend_obj()
         }
 
+        assert_eq!(self.name.as_str(), T::CLASS_NAME);
         self.object_override = Some(create_object::<T>);
         self
     }
@@ -282,7 +287,7 @@ impl ClassBuilder {
     ///
     /// Returns an [`Error`] variant if the class could not be registered.
     pub fn build(mut self) -> Result<&'static mut ClassEntry> {
-        self.ptr.name = ZendString::new_interned(&self.name)?.release();
+        self.ptr.name = ZendString::new_interned(&self.name, true)?.into_inner();
 
         self.methods.push(FunctionEntry::end());
         let func = Box::into_raw(self.methods.into_boxed_slice()) as *const FunctionEntry;
@@ -343,13 +348,19 @@ impl ClassBuilder {
 
 impl Debug for ClassEntry {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        let name: String = unsafe { ZendString::from_ptr(self.name, false) }
-            .and_then(|str| str.try_into())
-            .map_err(|_| std::fmt::Error)?;
+        let name: String = unsafe { self.name.as_ref() }
+            .and_then(|s| s.try_into().ok())
+            .ok_or(std::fmt::Error)?;
 
         f.debug_struct("ClassEntry")
             .field("name", &name)
             .field("flags", &self.flags())
+            .field("is_interface", &self.is_interface())
+            .field(
+                "interfaces",
+                &self.interfaces().map(|iter| iter.collect::<Vec<_>>()),
+            )
+            .field("parent", &self.parent())
             .finish()
     }
 }
