@@ -5,36 +5,64 @@ use std::{convert::TryFrom, fmt::Display};
 use crate::{
     bindings::{
         IS_ARRAY, IS_CALLABLE, IS_CONSTANT_AST, IS_DOUBLE, IS_FALSE, IS_LONG, IS_MIXED, IS_NULL,
-        IS_OBJECT, IS_REFERENCE, IS_RESOURCE, IS_STRING, IS_TRUE, IS_UNDEF, IS_VOID, _IS_BOOL,
+        IS_OBJECT, IS_PTR, IS_REFERENCE, IS_RESOURCE, IS_STRING, IS_TRUE, IS_UNDEF, IS_VOID,
+        _IS_BOOL,
     },
     errors::{Error, Result},
     php::flags::ZvalTypeFlags,
 };
 
 /// Valid data types for PHP.
-#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
-#[repr(u32)]
+#[derive(Debug, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 pub enum DataType {
-    Undef = IS_UNDEF,
+    Undef,
+    Null,
+    False,
+    True,
+    Long,
+    Double,
+    String,
+    Array,
+    Object(Option<&'static str>),
+    Resource,
+    Reference,
+    Callable,
+    ConstantExpression,
+    Void,
+    Mixed,
+    Bool,
+    Ptr,
+}
 
-    Null = IS_NULL,
-    False = IS_FALSE,
-    True = IS_TRUE,
-    Long = IS_LONG,
-    Double = IS_DOUBLE,
-    String = IS_STRING,
-    Array = IS_ARRAY,
-    Object = IS_OBJECT,
-    Resource = IS_RESOURCE,
-    Reference = IS_REFERENCE,
-    Callable = IS_CALLABLE,
+impl Default for DataType {
+    fn default() -> Self {
+        Self::Void
+    }
+}
 
-    ConstantExpression = IS_CONSTANT_AST,
-    Void = IS_VOID,
-    Mixed = IS_MIXED,
-
-    /// Only used when creating arguments.
-    Bool = _IS_BOOL,
+impl DataType {
+    /// Returns the integer representation of the data type.
+    pub const fn as_u32(&self) -> u32 {
+        match self {
+            DataType::Undef => IS_UNDEF,
+            DataType::Null => IS_NULL,
+            DataType::False => IS_FALSE,
+            DataType::True => IS_TRUE,
+            DataType::Long => IS_LONG,
+            DataType::Double => IS_DOUBLE,
+            DataType::String => IS_STRING,
+            DataType::Array => IS_ARRAY,
+            DataType::Object(_) => IS_OBJECT,
+            DataType::Resource => IS_RESOURCE,
+            DataType::Reference => IS_RESOURCE,
+            DataType::Callable => IS_CALLABLE,
+            DataType::ConstantExpression => IS_CONSTANT_AST,
+            DataType::Void => IS_VOID,
+            DataType::Mixed => IS_MIXED,
+            DataType::Bool => _IS_BOOL,
+            DataType::Ptr => IS_PTR,
+        }
+    }
 }
 
 // TODO: Ideally want something like this
@@ -69,25 +97,26 @@ impl TryFrom<ZvalTypeFlags> for DataType {
         contains!(Double);
         contains!(String);
         contains!(Array);
-        contains!(Object);
         contains!(Resource);
         contains!(Callable);
         contains!(ConstantExpression);
         contains!(Void);
 
+        if value.contains(ZvalTypeFlags::Object) {
+            return Ok(DataType::Object(None));
+        }
+
         Err(Error::UnknownDatatype(0))
     }
 }
 
-impl TryFrom<u32> for DataType {
-    type Error = Error;
-
+impl From<u32> for DataType {
     #[allow(clippy::bad_bit_mask)]
-    fn try_from(value: u32) -> Result<Self> {
+    fn from(value: u32) -> Self {
         macro_rules! contains {
             ($c: ident, $t: ident) => {
                 if (value & $c) == $c {
-                    return Ok(DataType::$t);
+                    return DataType::$t;
                 }
             };
         }
@@ -95,11 +124,8 @@ impl TryFrom<u32> for DataType {
         contains!(IS_VOID, Void);
         contains!(IS_CALLABLE, Callable);
         contains!(IS_CONSTANT_AST, ConstantExpression);
-        contains!(IS_CONSTANT_AST, ConstantExpression);
-        contains!(IS_CONSTANT_AST, ConstantExpression);
         contains!(IS_REFERENCE, Reference);
         contains!(IS_RESOURCE, Resource);
-        contains!(IS_OBJECT, Object);
         contains!(IS_ARRAY, Array);
         contains!(IS_STRING, String);
         contains!(IS_DOUBLE, Double);
@@ -107,9 +133,15 @@ impl TryFrom<u32> for DataType {
         contains!(IS_TRUE, True);
         contains!(IS_FALSE, False);
         contains!(IS_NULL, Null);
+        contains!(IS_PTR, Ptr);
+
+        if (value & IS_OBJECT) == IS_OBJECT {
+            return DataType::Object(None);
+        }
+
         contains!(IS_UNDEF, Undef);
 
-        Err(Error::UnknownDatatype(value))
+        DataType::Mixed
     }
 }
 
@@ -124,7 +156,7 @@ impl Display for DataType {
             DataType::Double => write!(f, "Double"),
             DataType::String => write!(f, "String"),
             DataType::Array => write!(f, "Array"),
-            DataType::Object => write!(f, "Object"),
+            DataType::Object(obj) => write!(f, "{}", obj.as_deref().unwrap_or("Object")),
             DataType::Resource => write!(f, "Resource"),
             DataType::Reference => write!(f, "Reference"),
             DataType::Callable => write!(f, "Callable"),
@@ -132,6 +164,7 @@ impl Display for DataType {
             DataType::Void => write!(f, "Void"),
             DataType::Bool => write!(f, "Bool"),
             DataType::Mixed => write!(f, "Mixed"),
+            DataType::Ptr => write!(f, "Pointer"),
         }
     }
 }
@@ -163,7 +196,7 @@ mod tests {
         test!(IS_DOUBLE, Double);
         test!(IS_STRING, String);
         test!(IS_ARRAY, Array);
-        test!(IS_OBJECT, Object);
+        assert_eq!(DataType::try_from(IS_OBJECT), Ok(DataType::Object(None)));
         test!(IS_RESOURCE, Resource);
         test!(IS_REFERENCE, Reference);
         test!(IS_CONSTANT_AST, ConstantExpression);
@@ -173,7 +206,7 @@ mod tests {
         test!(IS_INTERNED_STRING_EX, String);
         test!(IS_STRING_EX, String);
         test!(IS_ARRAY_EX, Array);
-        test!(IS_OBJECT_EX, Object);
+        assert_eq!(DataType::try_from(IS_OBJECT_EX), Ok(DataType::Object(None)));
         test!(IS_RESOURCE_EX, Resource);
         test!(IS_REFERENCE_EX, Reference);
         test!(IS_CONSTANT_AST_EX, ConstantExpression);
