@@ -81,9 +81,9 @@ impl ModuleBuilder {
                 info_func: None,
                 version: ptr::null(),
                 globals_size: 0,
-                #[cfg(not(feature = "zts"))]
+                #[cfg(not(php_zts))]
                 globals_ptr: ptr::null::<c_void>() as *mut c_void,
-                #[cfg(feature = "zts")]
+                #[cfg(php_zts)]
                 globals_id_ptr: ptr::null::<c_void>() as *mut crate::bindings::ts_rsrc_id,
                 globals_ctor: None,
                 globals_dtor: None,
@@ -178,3 +178,14 @@ impl ModuleEntry {
         Box::into_raw(Box::new(self))
     }
 }
+
+/// Called by startup functions registered with the `#[php_startup]` macro. Initializes all
+/// classes that are defined by ext-php-rs (i.e. [`Closure`]).
+///
+/// [`Closure`]: ext_php_rs::php::types::closure::Closure
+#[doc(hidden)]
+#[inline(always)]
+pub fn ext_php_rs_startup() {
+    #[cfg(feature = "closure")]
+    crate::php::types::closure::Closure::build();
+}
