@@ -1,18 +1,17 @@
 //! The base value in PHP. A Zval can contain any PHP type, and the type that it contains is
 //! determined by a property inside the struct. The content of the Zval is stored in a union.
 
-use core::slice;
 use std::{
-    collections::HashMap,
     convert::{TryFrom, TryInto},
+    ffi::c_void,
     fmt::Debug,
     ptr,
 };
 
 use crate::{
     bindings::{
-        _zval_struct__bindgen_ty_1, _zval_struct__bindgen_ty_2, ext_php_rs_zend_string_release,
-        zend_is_callable, zend_resource, zend_value, zval,
+        _zval_struct__bindgen_ty_1, _zval_struct__bindgen_ty_2, zend_is_callable, zend_resource,
+        zend_value, zval, zval_ptr_dtor,
     },
     errors::{Error, Result},
     php::pack::Pack,
@@ -24,7 +23,13 @@ use crate::php::{
     types::{long::ZendLong, string::ZendString},
 };
 
-use super::{array::ZendHashTable, callable::Callable, object::ZendObject};
+use super::{
+    array::{HashTable, OwnedHashTable},
+    callable::Callable,
+    object::ZendObject,
+    rc::PhpRc,
+    string::ZendStr,
+};
 
 /// Zend value. Represents most data types that are in the Zend engine.
 pub type Zval = zval;
@@ -32,15 +37,15 @@ pub type Zval = zval;
 unsafe impl Send for Zval {}
 unsafe impl Sync for Zval {}
 
-impl<'a> Zval {
+impl Zval {
     /// Creates a new, empty zval.
-    pub(crate) const fn new() -> Self {
+    pub const fn new() -> Self {
         Self {
             value: zend_value {
                 ptr: ptr::null_mut(),
             },
             u1: _zval_struct__bindgen_ty_1 {
-                type_info: DataType::Null as u32,
+                type_info: DataType::Null.as_u32(),
             },
             u2: _zval_struct__bindgen_ty_2 { next: 0 },
         }
@@ -75,6 +80,18 @@ impl<'a> Zval {
         }
     }
 
+    /// Returns the value of the zval as a zend string, if it is a string.
+    ///
+    /// Note that this functions output will not be the same as [`string()`](#method.string), as
+    /// this function does not attempt to convert other types into a [`String`].
+    pub fn zend_str(&self) -> Option<&ZendStr> {
+        if self.is_string() {
+            unsafe { self.value.str_.as_ref() }
+        } else {
+            None
+        }
+    }
+
     /// Returns the value of the zval if it is a string.
     ///
     /// If the zval does not contain a string, the function will check if it contains a
@@ -92,23 +109,8 @@ impl<'a> Zval {
     /// Note that this functions output will not be the same as [`string()`](#method.string), as
     /// this function does not attempt to convert other types into a [`String`], as it could not
     /// pass back a [`&str`] in those cases.
-    pub fn str(&'a self) -> Option<&'a str> {
-        if self.is_string() {
-            // SAFETY: Zend strings have a length that we know we can read.
-            // By reading this many bytes we will not run into any issues.
-            //
-            // We can safely cast our *const c_char into a *const u8 as both
-            // only occupy one byte.
-            unsafe {
-                std::str::from_utf8(slice::from_raw_parts(
-                    (*self.value.str_).val.as_ptr() as *const u8,
-                    (*self.value.str_).len as usize,
-                ))
-                .ok()
-            }
-        } else {
-            None
-        }
+    pub fn str(&self) -> Option<&str> {
+        self.zend_str().and_then(|zs| zs.as_str())
     }
 
     /// Returns the value of the zval if it is a string and can be unpacked into a vector of a
@@ -143,10 +145,19 @@ impl<'a> Zval {
         }
     }
 
-    /// Returns the value of the zval if it is an array.
-    pub fn array(&self) -> Option<ZendHashTable<'a>> {
+    /// Returns an immutable reference to the underlying zval hashtable if the zval contains an array.
+    pub fn array(&self) -> Option<&HashTable> {
         if self.is_array() {
-            unsafe { ZendHashTable::from_ptr(self.value.arr, false) }.ok()
+            unsafe { self.value.arr.as_ref() }
+        } else {
+            None
+        }
+    }
+
+    /// Returns a mutable reference to the underlying zval hashtable if the zval contains an array.
+    pub fn array_mut(&mut self) -> Option<&mut HashTable> {
+        if self.is_array() {
+            unsafe { self.value.arr.as_mut() }
         } else {
             None
         }
@@ -162,7 +173,16 @@ impl<'a> Zval {
     }
 
     /// Returns the value of the zval if it is a reference.
-    pub fn reference(&self) -> Option<&mut Zval> {
+    pub fn reference(&self) -> Option<&Zval> {
+        if self.is_reference() {
+            Some(&unsafe { self.value.ref_.as_ref() }?.val)
+        } else {
+            None
+        }
+    }
+
+    /// Returns a mutable reference to the underlying zval if it is a reference.
+    pub fn reference_mut(&mut self) -> Option<&mut Zval> {
         if self.is_reference() {
             Some(&mut unsafe { self.value.ref_.as_mut() }?.val)
         } else {
@@ -171,11 +191,25 @@ impl<'a> Zval {
     }
 
     /// Returns the value of the zval if it is callable.
-    pub fn callable(&'a self) -> Option<Callable<'a>> {
+    pub fn callable(&self) -> Option<Callable> {
         // The Zval is checked if it is callable in the `new` function.
         Callable::new(self).ok()
     }
 
+    /// Returns the value of the zval if it is a pointer.
+    ///
+    /// # Safety
+    ///
+    /// The caller must ensure that the pointer contained in the zval is in fact a pointer to an
+    /// instance of `T`, as the zval has no way of defining the type of pointer.
+    pub unsafe fn ptr<T>(&self) -> Option<*mut T> {
+        if self.is_ptr() {
+            Some(self.value.ptr as *mut T)
+        } else {
+            None
+        }
+    }
+
     /// Attempts to call the zval as a callable with a list of arguments to pass to the function.
     /// Note that a thrown exception inside the callable is not detectable, therefore you should
     /// check if the return value is valid rather than unwrapping. Returns a result containing the
@@ -191,28 +225,28 @@ impl<'a> Zval {
     }
 
     /// Returns the type of the Zval.
-    pub fn get_type(&self) -> Result<DataType> {
-        DataType::try_from(unsafe { self.u1.v.type_ } as u32)
+    pub fn get_type(&self) -> DataType {
+        DataType::from(unsafe { self.u1.v.type_ } as u32)
     }
 
     /// Returns true if the zval is a long, false otherwise.
     pub fn is_long(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Long as u8 }
+        self.get_type() == DataType::Long
     }
 
     /// Returns true if the zval is null, false otherwise.
     pub fn is_null(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Null as u8 }
+        self.get_type() == DataType::Null
     }
 
     /// Returns true if the zval is true, false otherwise.
     pub fn is_true(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::True as u8 }
+        self.get_type() == DataType::False
     }
 
     /// Returns true if the zval is false, false otherwise.
     pub fn is_false(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::False as u8 }
+        self.get_type() == DataType::False
     }
 
     /// Returns true if the zval is a bool, false otherwise.
@@ -222,32 +256,32 @@ impl<'a> Zval {
 
     /// Returns true if the zval is a double, false otherwise.
     pub fn is_double(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Double as u8 }
+        self.get_type() == DataType::Double
     }
 
     /// Returns true if the zval is a string, false otherwise.
     pub fn is_string(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::String as u8 }
+        self.get_type() == DataType::String
     }
 
     /// Returns true if the zval is a resource, false otherwise.
     pub fn is_resource(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Resource as u8 }
+        self.get_type() == DataType::Resource
     }
 
     /// Returns true if the zval is an array, false otherwise.
     pub fn is_array(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Array as u8 }
+        self.get_type() == DataType::Array
     }
 
     /// Returns true if the zval is an object, false otherwise.
     pub fn is_object(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Object as u8 }
+        matches!(self.get_type(), DataType::Object(_))
     }
 
     /// Returns true if the zval is a reference, false otherwise.
     pub fn is_reference(&self) -> bool {
-        unsafe { self.u1.v.type_ == DataType::Reference as u8 }
+        self.get_type() == DataType::Reference
     }
 
     /// Returns true if the zval is callable, false otherwise.
@@ -256,6 +290,11 @@ impl<'a> Zval {
         unsafe { zend_is_callable(ptr as *mut Self, 0, std::ptr::null_mut()) }
     }
 
+    /// Returns true if the zval contains a pointer, false otherwise.
+    pub fn is_ptr(&self) -> bool {
+        self.get_type() == DataType::Ptr
+    }
+
     /// Sets the value of the zval as a string. Returns nothing in a result when successful.
     ///
     /// # Parameters
@@ -263,21 +302,29 @@ impl<'a> Zval {
     /// * `val` - The value to set the zval as.
     /// * `persistent` - Whether the string should persist between requests.
     pub fn set_string(&mut self, val: &str, persistent: bool) -> Result<()> {
-        let zend_str = ZendString::new(val, persistent)?;
-        self.value.str_ = zend_str.release();
-        self.u1.type_info = ZvalTypeFlags::StringEx.bits();
+        self.set_zend_string(ZendString::new(val, persistent)?);
         Ok(())
     }
 
+    /// Sets the value of the zval as a Zend string.
+    ///
+    /// # Parameters
+    ///
+    /// * `val` - String content.
+    pub fn set_zend_string(&mut self, val: ZendString) {
+        self.change_type(ZvalTypeFlags::StringEx);
+        self.value.str_ = val.into_inner();
+    }
+
     /// Sets the value of the zval as a binary string, which is represented in Rust as a vector.
     ///
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_binary<T: Pack>(&mut self, val: Vec<T>) {
+        self.change_type(ZvalTypeFlags::StringEx);
         let ptr = T::pack_into(val);
         self.value.str_ = ptr;
-        self.u1.type_info = ZvalTypeFlags::StringEx.bits();
     }
 
     /// Sets the value of the zval as a interned string. Returns nothing in a result when successful.
@@ -285,10 +332,9 @@ impl<'a> Zval {
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
-    pub fn set_interned_string(&mut self, val: &str) -> Result<()> {
-        let zend_str = ZendString::new_interned(val)?;
-        self.value.str_ = zend_str.release();
-        self.u1.type_info = ZvalTypeFlags::InternedStringEx.bits();
+    /// * `persistent` - Whether the string should persist between requests.
+    pub fn set_interned_string(&mut self, val: &str, persistent: bool) -> Result<()> {
+        self.set_zend_string(ZendString::new_interned(val, persistent)?);
         Ok(())
     }
 
@@ -298,8 +344,12 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_long<T: Into<ZendLong>>(&mut self, val: T) {
-        self.value.lval = val.into();
-        self.u1.type_info = ZvalTypeFlags::Long.bits();
+        self._set_long(val.into())
+    }
+
+    fn _set_long(&mut self, val: ZendLong) {
+        self.change_type(ZvalTypeFlags::Long);
+        self.value.lval = val;
     }
 
     /// Sets the value of the zval as a double.
@@ -308,8 +358,12 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_double<T: Into<f64>>(&mut self, val: T) {
-        self.value.dval = val.into();
-        self.u1.type_info = ZvalTypeFlags::Double.bits();
+        self._set_double(val.into())
+    }
+
+    fn _set_double(&mut self, val: f64) {
+        self.change_type(ZvalTypeFlags::Double);
+        self.value.dval = val;
     }
 
     /// Sets the value of the zval as a boolean.
@@ -318,18 +372,22 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_bool<T: Into<bool>>(&mut self, val: T) {
-        self.u1.type_info = if val.into() {
-            DataType::True as u32
+        self._set_bool(val.into())
+    }
+
+    fn _set_bool(&mut self, val: bool) {
+        self.change_type(if val {
+            ZvalTypeFlags::True
         } else {
-            DataType::False as u32
-        };
+            ZvalTypeFlags::False
+        });
     }
 
     /// Sets the value of the zval as null.
     ///
     /// This is the default of a zval.
     pub fn set_null(&mut self) {
-        self.u1.type_info = ZvalTypeFlags::Null.bits();
+        self.change_type(ZvalTypeFlags::Null);
     }
 
     /// Sets the value of the zval as a resource.
@@ -338,7 +396,7 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_resource(&mut self, val: *mut zend_resource) {
-        self.u1.type_info = ZvalTypeFlags::ResourceEx.bits();
+        self.change_type(ZvalTypeFlags::ResourceEx);
         self.value.res = val;
     }
 
@@ -348,19 +406,39 @@ impl<'a> Zval {
     ///
     /// * `val` - The value to set the zval as.
     pub fn set_object(&mut self, val: &mut ZendObject) {
-        val.refcount_inc();
-        self.u1.type_info = ZvalTypeFlags::ObjectEx.bits();
+        self.change_type(ZvalTypeFlags::ObjectEx);
+        val.inc_count(); // TODO(david): not sure if this is needed :/
         self.value.obj = (val as *const ZendObject) as *mut ZendObject;
     }
 
-    /// Sets the value of the zval as an array. Returns nothng in a result on success.
+    /// Sets the value of the zval as an array. Returns nothing in a result on success.
     ///
     /// # Parameters
     ///
     /// * `val` - The value to set the zval as.
-    pub fn set_array(&mut self, val: ZendHashTable) {
-        self.u1.type_info = ZvalTypeFlags::ArrayEx.bits();
-        self.value.arr = val.into_ptr();
+    pub fn set_array<T: TryInto<OwnedHashTable, Error = Error>>(&mut self, val: T) -> Result<()> {
+        self.set_hashtable(val.try_into()?);
+        Ok(())
+    }
+
+    /// Sets the value of the zval as an array. Returns nothing in a result on success.
+    ///
+    /// # Parameters
+    ///
+    /// * `val` - The value to set the zval as.
+    pub fn set_hashtable(&mut self, val: OwnedHashTable) {
+        self.change_type(ZvalTypeFlags::ArrayEx);
+        self.value.arr = val.into_inner();
+    }
+
+    /// Sets the value of the zval as a pointer.
+    ///
+    /// # Parameters
+    ///
+    /// * `ptr` - The pointer to set the zval as.
+    pub fn set_ptr<T>(&mut self, ptr: *mut T) {
+        self.u1.type_info = ZvalTypeFlags::Ptr.bits();
+        self.value.ptr = ptr as *mut c_void;
     }
 
     /// Used to drop the Zval but keep the value of the zval intact.
@@ -369,8 +447,20 @@ impl<'a> Zval {
     /// will not be copied, but the pointer to the value (string for example) will be
     /// copied.
     pub(crate) fn release(mut self) {
+        // NOTE(david): don't use `change_type` here as we are wanting to keep the contents intact.
         self.u1.type_info = ZvalTypeFlags::Null.bits();
     }
+
+    /// Changes the type of the zval, freeing the current contents when applicable.
+    ///
+    /// # Parameters
+    ///
+    /// * `ty` - The new type of the zval.
+    fn change_type(&mut self, ty: ZvalTypeFlags) {
+        // SAFETY: we have exclusive mutable access to this zval so can free the contents.
+        unsafe { zval_ptr_dtor(self) };
+        self.u1.type_info = ty.bits();
+    }
 }
 
 impl Debug for Zval {
@@ -379,41 +469,45 @@ impl Debug for Zval {
         let ty = self.get_type();
         dbg.field("type", &ty);
 
-        if let Ok(ty) = ty {
-            macro_rules! field {
-                ($value: expr) => {
-                    dbg.field("val", &$value)
-                };
-            }
-
-            match ty {
-                DataType::Undef => field!(Option::<()>::None),
-                DataType::Null => field!(Option::<()>::None),
-                DataType::False => field!(false),
-                DataType::True => field!(true),
-                DataType::Long => field!(self.long()),
-                DataType::Double => field!(self.double()),
-                DataType::String | DataType::Mixed => field!(self.string()),
-                DataType::Array => field!(self.array()),
-                DataType::Object => field!(self.object()),
-                DataType::Resource => field!(self.resource()),
-                DataType::Reference => field!(self.reference()),
-                DataType::Callable => field!(self.string()),
-                DataType::ConstantExpression => field!(Option::<()>::None),
-                DataType::Void => field!(Option::<()>::None),
-                DataType::Bool => field!(self.bool()),
+        macro_rules! field {
+            ($value: expr) => {
+                dbg.field("val", &$value)
             };
         }
 
+        match ty {
+            DataType::Undef => field!(Option::<()>::None),
+            DataType::Null => field!(Option::<()>::None),
+            DataType::False => field!(false),
+            DataType::True => field!(true),
+            DataType::Long => field!(self.long()),
+            DataType::Double => field!(self.double()),
+            DataType::String | DataType::Mixed => field!(self.string()),
+            DataType::Array => field!(self.array()),
+            DataType::Object(_) => field!(self.object()),
+            DataType::Resource => field!(self.resource()),
+            DataType::Reference => field!(self.reference()),
+            DataType::Callable => field!(self.string()),
+            DataType::ConstantExpression => field!(Option::<()>::None),
+            DataType::Void => field!(Option::<()>::None),
+            DataType::Bool => field!(self.bool()),
+            // SAFETY: We are not accessing the pointer.
+            DataType::Ptr => field!(unsafe { self.ptr::<c_void>() }),
+        };
+
         dbg.finish()
     }
 }
 
 impl Drop for Zval {
     fn drop(&mut self) {
-        if self.is_string() {
-            unsafe { ext_php_rs_zend_string_release(self.value.str_) };
-        }
+        self.change_type(ZvalTypeFlags::Null);
+    }
+}
+
+impl Default for Zval {
+    fn default() -> Self {
+        Self::new()
     }
 }
 
@@ -446,6 +540,15 @@ pub trait IntoZval: Sized {
     fn set_zval(self, zv: &mut Zval, persistent: bool) -> Result<()>;
 }
 
+impl IntoZval for Zval {
+    const TYPE: DataType = DataType::Mixed;
+
+    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
+        *zv = self;
+        Ok(())
+    }
+}
+
 /// An object-safe version of the [`IntoZval`] trait.
 ///
 /// This trait is automatically implemented on any type that implements both [`IntoZval`] and [`Clone`].
@@ -564,7 +667,7 @@ try_into_zval_str!(String);
 try_into_zval_str!(&str);
 
 impl IntoZval for () {
-    const TYPE: DataType = DataType::Null;
+    const TYPE: DataType = DataType::Void;
 
     fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
         zv.set_null();
@@ -589,52 +692,18 @@ where
     }
 }
 
-impl<'a> IntoZval for ZendHashTable<'a> {
-    const TYPE: DataType = DataType::Array;
-
-    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
-        zv.set_array(self);
-        Ok(())
-    }
-}
-
-impl<T> IntoZval for Vec<T>
-where
-    T: IntoZval,
-{
-    const TYPE: DataType = DataType::Array;
-
-    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
-        let hm = self
-            .try_into()
-            .map_err(|_| Error::ZvalConversion(DataType::Array))?;
-        zv.set_array(hm);
-        Ok(())
-    }
-}
-
-impl<K, V> IntoZval for HashMap<K, V>
-where
-    K: AsRef<str>,
-    V: IntoZval,
-{
-    const TYPE: DataType = DataType::Array;
-
-    fn set_zval(self, zv: &mut Zval, _: bool) -> Result<()> {
-        let hm = self.try_into()?;
-        zv.set_array(hm);
-        Ok(())
-    }
-}
-
 /// Allows zvals to be converted into Rust types in a fallible way. Reciprocal of the [`IntoZval`]
 /// trait.
-///
-/// This trait requires the [`TryFrom`] trait to be implemented. All this trait does is contain the
-/// type of data that is expected when parsing the value, which is used when parsing arguments.
-pub trait FromZval<'a>: TryFrom<&'a Zval> {
+pub trait FromZval<'a>: Sized {
     /// The corresponding type of the implemented value in PHP.
     const TYPE: DataType;
+
+    /// Attempts to retrieve an instance of `Self` from a reference to a [`Zval`].
+    ///
+    /// # Parameters
+    ///
+    /// * `zval` - Zval to get value from.
+    fn from_zval(zval: &'a Zval) -> Option<Self>;
 }
 
 impl<'a, T> FromZval<'a> for Option<T>
@@ -642,32 +711,19 @@ where
     T: FromZval<'a>,
 {
     const TYPE: DataType = T::TYPE;
-}
 
-// Converting to an option is infallible.
-impl<'a, T> From<&'a Zval> for Option<T>
-where
-    T: FromZval<'a>,
-{
-    fn from(val: &'a Zval) -> Self {
-        val.try_into().ok()
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
+        Some(T::from_zval(zval))
     }
 }
 
 macro_rules! try_from_zval {
     ($type: ty, $fn: ident, $dt: ident) => {
-        impl<'a> FromZval<'a> for $type {
+        impl FromZval<'_> for $type {
             const TYPE: DataType = DataType::$dt;
-        }
 
-        impl TryFrom<&Zval> for $type {
-            type Error = Error;
-
-            fn try_from(value: &Zval) -> Result<Self> {
-                value
-                    .$fn()
-                    .and_then(|val| val.try_into().ok())
-                    .ok_or(Error::ZvalConversion(value.get_type()?))
+            fn from_zval(zval: &Zval) -> Option<Self> {
+                zval.$fn().and_then(|val| val.try_into().ok())
             }
         }
 
@@ -675,7 +731,7 @@ macro_rules! try_from_zval {
             type Error = Error;
 
             fn try_from(value: Zval) -> Result<Self> {
-                (&value).try_into()
+                Self::from_zval(&value).ok_or(Error::ZvalConversion(value.get_type()))
             }
         }
     };
@@ -698,126 +754,27 @@ try_from_zval!(f64, double, Double);
 try_from_zval!(bool, bool, Bool);
 try_from_zval!(String, string, String);
 
-impl<'a> FromZval<'a> for f32 {
+impl FromZval<'_> for f32 {
     const TYPE: DataType = DataType::Double;
-}
 
-impl<'a> TryFrom<&'a Zval> for f32 {
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        value
-            .double()
-            .map(|v| v as f32)
-            .ok_or(Error::ZvalConversion(value.get_type()?))
+    fn from_zval(zval: &Zval) -> Option<Self> {
+        zval.double().map(|v| v as f32)
     }
 }
 
 impl<'a> FromZval<'a> for &'a str {
     const TYPE: DataType = DataType::String;
-}
 
-impl<'a> TryFrom<&'a Zval> for &'a str {
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        value.str().ok_or(Error::ZvalConversion(value.get_type()?))
-    }
-}
-
-impl<'a> FromZval<'a> for ZendHashTable<'a> {
-    const TYPE: DataType = DataType::Array;
-}
-
-impl<'a> TryFrom<&'a Zval> for ZendHashTable<'a> {
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        value
-            .array()
-            .ok_or(Error::ZvalConversion(value.get_type()?))
-    }
-}
-
-impl<'a, T> FromZval<'a> for Vec<T>
-where
-    T: FromZval<'a>,
-{
-    const TYPE: DataType = DataType::Array;
-}
-
-impl<'a, T> TryFrom<&'a Zval> for Vec<T>
-where
-    T: FromZval<'a>,
-{
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        value
-            .array()
-            .ok_or(Error::ZvalConversion(value.get_type()?))?
-            .try_into()
-    }
-}
-
-impl<'a, T> TryFrom<Zval> for Vec<T>
-where
-    T: FromZval<'a>,
-{
-    type Error = Error;
-
-    fn try_from(value: Zval) -> Result<Self> {
-        value
-            .array()
-            .ok_or(Error::ZvalConversion(value.get_type()?))?
-            .try_into()
-    }
-}
-
-impl<'a, T> FromZval<'a> for HashMap<String, T>
-where
-    T: FromZval<'a>,
-{
-    const TYPE: DataType = DataType::Array;
-}
-
-impl<'a, T> TryFrom<&'a Zval> for HashMap<String, T>
-where
-    T: FromZval<'a>,
-{
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        value
-            .array()
-            .ok_or(Error::ZvalConversion(value.get_type()?))?
-            .try_into()
-    }
-}
-
-impl<'a, T> TryFrom<Zval> for HashMap<String, T>
-where
-    T: FromZval<'a>,
-{
-    type Error = Error;
-
-    fn try_from(value: Zval) -> Result<Self> {
-        value
-            .array()
-            .ok_or(Error::ZvalConversion(value.get_type()?))?
-            .try_into()
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
+        zval.str()
     }
 }
 
 impl<'a> FromZval<'a> for Callable<'a> {
     const TYPE: DataType = DataType::Callable;
-}
 
-impl<'a> TryFrom<&'a Zval> for Callable<'a> {
-    type Error = Error;
-
-    fn try_from(value: &'a Zval) -> Result<Self> {
-        Callable::new(value)
+    fn from_zval(zval: &'a Zval) -> Option<Self> {
+        Callable::new(zval).ok()
     }
 }
 
